/* dcf_connsequence.c
 from dc fighting v0.2.2
  by
   jimbo
    Dan Potter and Jordan DeLong of Cryptic Allusion
     Andrew K.
      Marcus Comstedt


 the rules of the game...
  consequence function:
   void determine_result( fighter_t *p1, fighter_t *p2 );
    makes appropriate changes to players when conditions dictate:
     i.e. during a successful attack
 
 
 
      p1       p2         result
      -------------------------------
      attack   --         p2 hit; add anchor fall_down; subtract energy...
      attack   block      --
      attack   counter    p1 hit; 
      
      throw    block      p2 hit
      throw    throw      --
      throw    attack     p1 hit
      throw    counter    p2 hit

      states that demand action are ATTACK, THROW


v0.2.4: adding height states to fighting logic

 */


/* this consequence function is called once per main loop as follows:
 *    determine_result( player1, player1->opponent );
 *
 */
void determine_result( fighter_t *p1, fighter_t *p2 ) {
  int i;
  float attack_distance;
  int throw = 0;

  /*** v0.1.9 - function scoped ***/
  double fighting_distance;
  double fighting_vector[4];


  /* v0.1.7: win condition
   *    - reset
   *
   *
   *
   */
  if ( p1->energy <= 0 ) {

    if ( running_state == RUNSTATE_FIGHT )
      p2->victories++;

    running_state = RUNSTATE_VICTORY;

#if 0
    //    printf("WINNER!\n");
    p1->default_state = FIGHT_STAND;
    p1->default_state = FIGHT_DOWN;
    p2->fighting_state = FIGHT_DOWN;
    // timestep = 0.0;

    /*** result: p2 lost - clear out p2's future ***/
    for ( i=0; i<FUTURE_SIZE; i++ ) {
      p1->future_lists[0][i] = 0.0;
      p2->future_lists[0][i] = 0.0;
    }
    add_anchor( p2, 0.15, fallen_down, 10 );
    add_anchor( p2, 0.35, lying_down, 10 );
#endif

  }




  /* v0.1.6: multiple-height attack logic
     int power, height, range;
     int power2, height2, range2;
   *
   * use integer math:
   * power = num / 100;
   * height = (num/10) % 10
   * range = num % 10;
   *
   *
   *
   *
   *
   */ 
  p1->power = 0;
  p1->range = 0;
  p1->height = 0;
  p1->power = (p1->fighting_state / 1000);// * 1000;
  p1->range = ( (p1->fighting_state / 100) % 10 );// * 100;
  p1->height = ((p1->fighting_state / 10) % 10) * 10;
  
  
  /*** check for range first ***/
  switch( p1->range ) {
  case FIGHT_THROW/100:        attack_distance = throw_distance;   break;
  case FIGHT_SHORT_RANGE/100:  attack_distance = short_distance;   break;
  case FIGHT_MEDIUM_RANGE/100: attack_distance = medium_distance;  break;
  case FIGHT_LONG_RANGE/100:   attack_distance = long_distance;    break;
  default:                     attack_distance = default_distance; break;
  }


  /* v0.1.9 : fighting distance calculation
   *
   *  v0.2.1 - need to change this...
   *   v0.2.2: did
   *
   */

  fighting_vector[0] = p2->position[0] - p1->position[0];
  fighting_vector[1] = p2->position[1] - p1->position[1];
  fighting_vector[2] = p2->position[2] - p1->position[2];




  fighting_distance = sqrt ( fighting_vector[0] * fighting_vector[0] + 
			     fighting_vector[1] * fighting_vector[1] + 
			     fighting_vector[2] * fighting_vector[2]
			     );




  /* v0.2.2: miss condition:
   *   this block will not count an attack that misses its intended target...
   *    players' rotations are contained in the opponent_rotation struct member variable.
   *     compare this value with the actual direction to the opponent, add a tolerance, you have your answer.
   *
   *  empirically, a hit is when opponent_rotation == - atan2(x,z) of the p1p2 vector.
   *
   */


    if ( fabs( 
	      p1->opponent_rotation 
	      + atan2(fighting_vector[0],fighting_vector[2]) 
	      )
	 > attack_direction_tolerance ) { 
      return;
    }





  /*** fighters are within range ***/
  if ( fighting_distance < attack_distance ) {



    /*   throw vs. nothing or block or counter
     *    p1        p2
     *
     *    == p2 thrown
     *
     */
    if ( ( p1->fighting_state >= FIGHT_ATTACK + FIGHT_THROW ) 
	 && ( p2->fighting_state < FIGHT_ATTACK + FIGHT_THROW ) ) {
      
      /*** clear out p2's future ***/
      for ( i=0; i<FUTURE_SIZE; i++ ) {
	p2->future_lists[0][i] = 0.0;
      }

      /*** v0.1.7: count the hit in energy (want to scale by power) ***/
      p2->energy -= 5 * p1->power;
      
      /*** should a hit count again or not? already got credit for the hit... ***/
      p1->fighting_state = FIGHT_MOVE;
      
      do_move( p2, MOVE_BE_THROWN );

      p2->position[0] += 2.0 * fighting_vector[0];
      p2->position[1] += 2.0 * fighting_vector[1];
      p2->position[2] += 2.0 * fighting_vector[2];

    }
    
    
    



    /* attack vs. block
     *   p1         p2
     *
     *  == p2 hurt slightly, knocked back slightly
     *
     *  adding direction calculation v0.2.2:
     *   cant block a hit youre not facing
     *
     */
    else if ( ( p1->power > 1 ) 
	      && 
	      ( p2->fighting_state == FIGHT_DEFENSE + FIGHT_BLOCK  ) 
	     ) {


      /* v0.2.2: facing right direction for block...
       *   note the minus sign - negative to attack; other direction
       */
      if ( fabs( 
	     p2->opponent_rotation 
	     - atan2(fighting_vector[0],fighting_vector[2]) 
	     )
	< block_direction_tolerance 
	) {
	
	if ( block_damage == 1 ) 
	    p2->energy -= 1;
	
	/*** should e hit count again or not? already got credit for the hit... ***/
	p1->fighting_state = FIGHT_MOVE;
	

	/*** v0.1.9 : move fighter back from impact ***/
	p2->position[0] += 0.02 * fighting_vector[0];
	p2->position[1] += 0.02 * fighting_vector[1];
	p2->position[2] += 0.02 * fighting_vector[2];
      }
    


      /* missed the block - regular hit
       *  this would be nicer if it was the same code as below, for consistency...
       * 
       */
      else {


	/*** result: p2 hit - clear out p2's future ***/
	for ( i=0; i<FUTURE_SIZE; i++ ) {
	  p2->future_lists[0][i] = 0.0;
	}
      
	/*** v0.1.7: count the hit in energy (want to scale by power) ***/
	p2->energy -= 1 * p1->power;
      
	/*** should e hit count again or not? already got credit for the hit... ***/
	p1->fighting_state = FIGHT_MOVE;
      

	switch( p1->height ) {
	case FIGHT_HIGH: do_move( p2, MOVE_BE_HIT_HIGH ); break;
	case FIGHT_MIDDLE: do_move( p2, MOVE_BE_HIT_MIDDLE ); break;
	  //case FIGHT_LOW: do_move( p2, MOVE_BE_HIT_LOW ); break;
	default: do_move( p2, MOVE_BE_HIT_HIGH ); break;
	};


	p2->position[0] += 0.2 * fighting_vector[0];
	p2->position[1] += 0.2 * fighting_vector[1];
	p2->position[2] += 0.2 * fighting_vector[2];
      }


    }






    /* attack vs. walking or standing : attack wins 
     *   p1         p2
     * 
     *   == p2 attacked
     */
    else if ( ( p1->power > 1 ) 
	       && ( p2->fighting_state <= FIGHT_STAND  ) ) {
      


      /*** result: p2 hit - clear out p2's future ***/
      for ( i=0; i<FUTURE_SIZE; i++ ) {
	p2->future_lists[0][i] = 0.0;
      }
      
      /*** v0.1.7: count the hit in energy (want to scale by power) ***/
      p2->energy -= 1 * p1->power;
      
      /*** should e hit count again or not? already got credit for the hit... ***/
      p1->fighting_state = FIGHT_MOVE;
      


      switch( p1->height ) {
      case FIGHT_HIGH: do_move( p2, MOVE_BE_HIT_HIGH ); break;
      case FIGHT_MIDDLE: do_move( p2, MOVE_BE_HIT_MIDDLE ); break;
	//case FIGHT_LOW: do_move( p2, MOVE_BE_HIT_LOW ); break;
      default: do_move( p2, MOVE_BE_HIT_HIGH ); break;
      };


      p2->position[0] += 0.2 * fighting_vector[0];
      p2->position[1] += 0.2 * fighting_vector[1];
      p2->position[2] += 0.2 * fighting_vector[2];



    }














    /* attack vs. counter/parry
     *   p1         p2
     *
     *   == p1 hit
     */
    else if ( ( p1->power > 1 ) 
	      && ( p2->fighting_state == FIGHT_DEFENSE + FIGHT_COUNTER  ) ) {

      /*** result: p2 hit - clear out p2's future ***/
      for ( i=0; i<FUTURE_SIZE; i++ ) {
	p1->future_lists[0][i] = 0.0;
      }

      /*** v0.1.7: count the hit in energy (want to scale by power) ***/
      p1->energy -= 3 * p2->power;

      /*** should e hit count again or not? already got credit for the hit... ***/
      p2->fighting_state = FIGHT_MOVE;

      /*** if attack, change to stand ***/
      if ( p1->fighting_state > FIGHT_STAND ) { p1->fighting_state = FIGHT_STAND; }


      do_move( p1, MOVE_BE_COUNTERED );


      p1->position[0] -= 0.5 * fighting_vector[0];
      p1->position[1] -= 0.5 * fighting_vector[1];
      p1->position[2] -= 0.5 * fighting_vector[2];
    }





      /* throw vs. attack
       *
       *  - not sure if this is correct... which one taks precedence?
       *
       *
       */
    else if (0) {
    }




      /*
       *
       *
       */
    else if (0) {
    }




  }






}

